# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/plugin.states.ipynb (unless otherwise specified).

__all__ = ['RUN_IDLE', 'RUN_INITIALIZED', 'RUN_USER_ACTION_NEEDED', 'RUN_USER_ACTION_COMPLETED', 'RUN_STARTED',
           'RUN_FAILED', 'RUN_COMPLETED', 'RUN_ABORTED', 'RUN_AUTHENTICATED']

# Cell

# run states
RUN_IDLE = 'idle'
RUN_INITIALIZED = 'initialized'
RUN_USER_ACTION_NEEDED = 'userActionNeeded'
RUN_USER_ACTION_COMPLETED = 'ready'
RUN_STARTED = 'started'
RUN_FAILED = 'error'
RUN_COMPLETED = 'done'
RUN_ABORTED = 'aborted'
RUN_AUTHENTICATED = 'authenticated'