# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"read_file": "basic.ipynb",
         "read_json": "basic.ipynb",
         "write_json": "basic.ipynb",
         "unzip": "basic.ipynb",
         "Path.ls": "basic.ipynb",
         "PYI_HOME": "basic.ipynb",
         "PYI_TESTDATA": "basic.ipynb",
         "HOME_DIR": "basic.ipynb",
         "MODEL_DIR": "basic.ipynb",
         "MEMRI_S3": "basic.ipynb",
         "CVU_BASE_PATH": "cvu.utils.ipynb",
         "get_default_cvu": "cvu.utils.ipynb",
         "list_default_cvus": "cvu.utils.ipynb",
         "filter_rows": "data.dataset.ipynb",
         "Dataset": "data.dataset.ipynb",
         "MEMRI_PATH": "data.loader.ipynb",
         "MEMRI_GITLAB_BASE_URL": "data.loader.ipynb",
         "ACCESS_TOKEN_PATH": "data.loader.ipynb",
         "GITLAB_API_BASE_URL": "data.loader.ipynb",
         "DEFAULT_PLUGIN_MODEL_PACKAGE_NAME": "data.loader.ipynb",
         "DEFAULT_PYTORCH_MODEL_NAME": "data.loader.ipynb",
         "DEFAULT_HUGGINFACE_CONFIG_NAME": "data.loader.ipynb",
         "DEFAULT_PACKAGE_VERSION": "data.loader.ipynb",
         "TIME_FORMAT_GITLAB": "data.loader.ipynb",
         "PROJET_ID_PATTERN": "data.loader.ipynb",
         "find_git_repo": "data.loader.ipynb",
         "get_registry_api_key": "data.loader.ipynb",
         "upload_in_chunks": "data.loader.ipynb",
         "IterableToFileAdapter": "data.loader.ipynb",
         "write_file_to_package_registry": "data.loader.ipynb",
         "project_id_from_name": "data.loader.ipynb",
         "get_project_id_from_project_path_unsafe": "data.loader.ipynb",
         "write_huggingface_model_to_package_registry": "data.loader.ipynb",
         "write_model_to_package_registry": "data.loader.ipynb",
         "download_package_file": "data.loader.ipynb",
         "download_huggingface_model_for_project": "data.loader.ipynb",
         "load_huggingface_model_for_project": "data.loader.ipynb",
         "DEFAULT_ENCODING": "data.photo.ipynb",
         "show_images": "data.photo.ipynb",
         "Photo": "data.photo.ipynb",
         "Query": "exporters.exporters.ipynb",
         "ALL_EDGES": "itembase.ipynb",
         "Edge": "itembase.ipynb",
         "EdgeList": "itembase.ipynb",
         "T": "itembase.ipynb",
         "ItemBase": "itembase.ipynb",
         "Item": "itembase.ipynb",
         "PLUGIN_DIR": "plugin.authenticators.credentials.ipynb",
         "read_username_password": "plugin.authenticators.credentials.ipynb",
         "OAuthAuthenticator": "plugin.authenticators.oauth.ipynb",
         "StatusListener": "plugin.listeners.ipynb",
         "force_exit_callback": "plugin.listeners.ipynb",
         "get_abort_plugin_listener": "plugin.listeners.ipynb",
         "POD_FULL_ADDRESS_ENV": "plugin.pluginbase.ipynb",
         "POD_TARGET_ITEM_ENV": "plugin.pluginbase.ipynb",
         "POD_OWNER_KEY_ENV": "plugin.pluginbase.ipynb",
         "POD_AUTH_JSON_ENV": "plugin.pluginbase.ipynb",
         "POD_PLUGIN_DNS_ENV": "plugin.pluginbase.ipynb",
         "PluginBase": "plugin.pluginbase.ipynb",
         "PluginError": "plugin.pluginbase.ipynb",
         "ExamplePlugin": "plugin.pluginbase.ipynb",
         "write_run_info": "plugin.pluginbase.ipynb",
         "get_plugin_cls": "plugin.pluginbase.ipynb",
         "run_plugin_from_run_id": "plugin.pluginbase.ipynb",
         "store_keys": "plugin.pluginbase.ipynb",
         "parse_metadata": "plugin.pluginbase.ipynb",
         "parse_config": "plugin.pluginbase.ipynb",
         "create_run_expanded": "plugin.pluginbase.ipynb",
         "run_plugin": "plugin.pluginbase.ipynb",
         "simulate_run_plugin_from_frontend": "plugin.pluginbase.ipynb",
         "RUN_IDLE": "plugin.states.ipynb",
         "RUN_INITIALIZED": "plugin.states.ipynb",
         "RUN_USER_ACTION_NEEDED": "plugin.states.ipynb",
         "RUN_USER_ACTION_COMPLETED": "plugin.states.ipynb",
         "RUN_STARTED": "plugin.states.ipynb",
         "RUN_FAILED": "plugin.states.ipynb",
         "RUN_COMPLETED": "plugin.states.ipynb",
         "RUN_ABORTED": "plugin.states.ipynb",
         "RUN_AUTHENTICATED": "plugin.states.ipynb",
         "PasswordAuthenticator": "plugins.authenticators.password.ipynb",
         "set_account_credentials": "plugins.authenticators.password.ipynb",
         "simulate_enter_credentials": "plugins.authenticators.password.ipynb",
         "DEFAULT_POD_ADDRESS": "pod.api.ipynb",
         "POD_VERSION": "pod.api.ipynb",
         "PodError": "pod.api.ipynb",
         "PodAPI": "pod.api.ipynb",
         "PodClient": "pod.client.ipynb",
         "Dog": "pod.client.ipynb",
         "Priority": "pod.db.ipynb",
         "DB": "pod.db.ipynb",
         "PYMEMRI_FOLDER": "pod.utils.ipynb",
         "POD_KEYS_FOLDER": "pod.utils.ipynb",
         "POD_KEYS_FILENAME": "pod.utils.ipynb",
         "POD_KEYS_FULL_FOLDER": "pod.utils.ipynb",
         "DEFAULT_POD_KEY_PATH": "pod.utils.ipynb",
         "read_pod_key": "pod.utils.ipynb",
         "get_params": "template.config.ipynb",
         "identifier_to_displayname": "template.config.ipynb",
         "get_param_config": "template.config.ipynb",
         "ALLOWED_TYPES": "template.config.ipynb",
         "create_config": "template.config.ipynb",
         "create_plugin_config": "template.config.ipynb",
         "TEMPLATE_URL": "template.formatter.ipynb",
         "TEMPLATE_BASE_PATH": "template.formatter.ipynb",
         "get_remote_url": "template.formatter.ipynb",
         "infer_git_info": "template.formatter.ipynb",
         "GITLAB_GROUPS": "template.formatter.ipynb",
         "str_to_identifier": "template.formatter.ipynb",
         "reponame_to_displayname": "template.formatter.ipynb",
         "download_plugin_template": "template.formatter.ipynb",
         "get_templates": "template.formatter.ipynb",
         "TemplateFormatter": "template.formatter.ipynb",
         "get_template_replace_dict": "template.formatter.ipynb",
         "plugin_from_template": "template.formatter.ipynb",
         "get_ci_variables": "test_utils.ipynb"}

modules = ["data/basic.py",
           "cvu/utils.py",
           "data/dataset.py",
           "data/loader.py",
           "data/photo.py",
           "exporters/exporters.py",
           "data/itembase.py",
           "plugin/authenticators/credentials.py",
           "plugin/authenticators/oauth.py",
           "plugin/listeners.py",
           "plugin/pluginbase.py",
           "plugin/states.py",
           "plugin/authenticators/password.py",
           "pod/api.py",
           "pod/client.py",
           "pod/db.py",
           "pod/utils.py",
           "template/config.py",
           "template/formatter.py",
           "test_utils.py"]

doc_url = "http://memri.docs.memri.io/pymemri/"

git_url = "https://gitlab.memri.io/memri/pymemri/tree/prod/"

def custom_doc_links(name): return None
