# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/test_utils.ipynb (unless otherwise specified).

__all__ = ['get_ci_variables']

# Cell
#hide
import os

def get_ci_variables(*varnames):
    """
    Gets ci variables from environment, can be set on gitlab in settings -> ci/cd -> variables
    """
    return {varname: os.environ.get(varname) for varname in varnames}