# For backwards compatibility with plugins, import from central schema here
# TODO remove this file and update plugins.

__all__ = ["Account", "PluginRun"]

from ..data.schema import Account, PluginRun