# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/plugin.authenticators.credentials.ipynb (unless otherwise specified).

__all__ = ['PLUGIN_DIR', 'read_username_password']

# Cell
from ...data.basic import read_json, HOME_DIR
PLUGIN_DIR = HOME_DIR / ".pymemri" / "plugins"

# Cell
def read_username_password(plugin: str):
    credentials_path = PLUGIN_DIR / plugin / "credentials.json"
    credentials = read_json(credentials_path)
    username, password = credentials["username"], credentials["password"]
    return username,password